DrawingBoard = LibStub("AceAddon-3.0"):NewAddon("DrawingBoard", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0", "AceSerializer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("DrawingBoard")


DrawingBoard.debug = true
DrawingBoard.version = 0.6


function DrawingBoard:OnEnable()
    -- Called when the addon is enabled
   local defaults = { profile = {
									bc_title = L["New Board"],
									bc_prefix = L["Assignment "],
									bc_channel = "raid",
									boards = {}
								}
					 }
    self.db = LibStub("AceDB-3.0"):New("DrawingBoardDB", defaults, "Default")
    
    -- self.boards contains the saved boards
    -- as board -> title, prefix, channel, tags[]
    -- tag in tags[] -> id, text, x, y, anchor, p_anchor, attached_to
    
    
    self.frames = {}
    self.tagframes = {}
    
    self.boards =  self.db.profile.boards
    
    self:CreateMainFrames()
	self:CreateBoardButtons()
    
    -- Select first board
    local i
    for number, _ in pairs(self.boards) do
        if not i then i = number end
         i = math.min(i,number)
    end
    self.active = i
    if not self.active then
		self:ShowBoard(self:NewBoard())
    else
		self:ShowBoard(self.active)
	end
    self:Close()
	
	self:RegisterComm("DrawingBoard", DrawingBoard.OnRecieve)
    self:RegisterChatCommand("db", "Open")
	self:RegisterEvent("RAID_ROSTER_UPDATE", DrawingBoard.Update)
	self:RegisterEvent("PLAYER_ENTERING_WORLD", DrawingBoard.Update)
end




-- ********* Comm functions
function DrawingBoard:OnRecieve(message, distribution, sender)
	local self = DrawingBoard
	local success, t = self:Deserialize(message)
	if not success then self:Debug("Something went wrong on message recieval") end
	if t.type == "ping" then
		local s = {type = "pong", version = self.version}
		self:SendCommMessage("DrawingBoard", self:Serialize(s), "WHISPER", sender)
	end
	if t.type == "pong" then
		if sender ~= UnitName("player") then
			table.insert(self.installs, {name = sender, version = t.version})
		end
	end
	if t.type == "board" then
		local found = false
		local id
		for i, board in pairs(self.boards) do
			if board.title == sender..":"..t.board.title then
				found = true
				id = i
			end
		end
		if not found then 
			id = self:NewBoard()
			self:Print(sender, L["has sent a board to you."])
		else
			self:Print(sender, L["has updated a board."])
		end
		self.boards[id] = t.board
		self:RenameBoard(id,  sender..":"..t.board.title)
		if self.active == id then
			self:UpdateActiveBoard()
		end
	end
end

function DrawingBoard:Update()
	local self = DrawingBoard
	self:UpdateActiveBoard()
	self:UpdateInstalls()
end

function DrawingBoard:UpdateInstalls()
	local t = {type = "ping"}
	self.installs = {}
	self:SendCommMessage("DrawingBoard", self:Serialize(t), "GUILD")
end

function DrawingBoard:SendBoard(boardID, distribution, target)
	local t = {type = "board", board = self.boards[boardID]}
	self:SendCommMessage("DrawingBoard", self:Serialize(t), distribution, target)
end


-- ********* Window functions

function DrawingBoard:Open() 
    for _, frame in pairs(self.frames) do
		frame:Show()
    end
    for _, frame in pairs(self.tagframes) do
		frame:Show()
    end
end


function DrawingBoard:Close()
    for _, frame in pairs(self.frames) do
		frame:Hide()
    end
    for _, frame in pairs(self.tagframes) do
		frame:Hide()
    end
end


-- ******** Board Functions

function DrawingBoard:HideActiveBoard()
    for id, frame in pairs(self.tagframes) do
        self:DeleteButton(id)
    end
	if self.frames["DrawingBoardBoard".. self.active.."Button"] then
		self.frames["DrawingBoardBoard".. self.active.."Button"]:UnlockHighlight()
	end
end


function DrawingBoard:DeleteBoard()
    self.frames["DrawingBoardBoard"..self.active.."Button"]:Hide()
    self.frames["DrawingBoardBoard"..self.active.."Button"] = nil
    
    self:HideActiveBoard()
    
    local i = 0
    local j = 100000
    for number, _ in pairs(self.boards) do
		if self.active > number then i = math.max(i, number) end
		if self.active < number then j = math.min(j, number) end
    end
    self.boards[self.active] = nil
    if i == 0 and j == 100000 then
		self:ShowBoard(self:NewBoard())
    elseif i>0 then
		self:ShowBoard(i)
    else
		self:ShowBoard(j)
    end
    self:SortBoardButtons()
end





function DrawingBoard:UpdateActiveBoard()
	local self = DrawingBoard
	if not self.active then return end
	
	for id, button in pairs(self.tagframes) do
		self:UpdateButton(id)
	end
	
	for i=1,8 do 
		if not self:HasButton("{rt"..i.."}") then
			self:CreateButton("{rt"..i.."}")
		end
	end
	
	if GetNumRaidMembers() > 0 then
		-- Add missing buttons
		for i=1,GetNumRaidMembers() do
			local name = UnitName("raid"..i)
			if not self:HasButton(name) then
				self:CreateButton(name)
			end
		end
	end
end


function DrawingBoard:NewBoard()
    -- Chose new board number
    local number = 0
    for board_num,_  in pairs(self.boards) do
		number = math.max(number, board_num)
    end
    number = number + 1
    
    -- Create Empty Board
    local board = {}
    board.prefix = self.db.profile.bc_prefix
    board.channel = self.db.profile.bc_channel
    board.number = number
    board.tags = {}
    self.boards[number] = board  
   
    -- Create Board Button
    local b = CreateFrame("Button", "DrawingBoardBoard"..number.."Button", f, "OptionsButtonTemplate")
    b:SetFrameStrata("FULLSCREEN")
    b:SetScript("OnClick", function () DrawingBoard:ShowBoard(number) end)
	b:Show()
    self.frames["DrawingBoardBoard"..number.."Button"] = b
    
	self:SortBoardButtons()
    self:RenameBoard(number, self.db.profile.bc_title)
    return number
end






function DrawingBoard:ShowBoard(boardnumber)
	local self = DrawingBoard
    if self.active then
		self:HideActiveBoard()
    end
	self.active = boardnumber
    for _, tag in pairs(self.boards[boardnumber].tags) do
		self:CreateButton(tag.text, tag.id)
	end
	
	for _, tag in pairs(self.boards[boardnumber].tags) do
        if not tag.attached_to then
			self.tagframes[tag.id]:SetScale(1.5)
			self.tagframes[tag.id]:SetRelativePoint(tag.x, tag.y)
        end
    end
    for _, tag in pairs(self.boards[boardnumber].tags) do
        if tag.attached_to then
			self:AttachButton(tag.id, tag.attached_to)
        end
    end
	
	self:UpdateActiveBoard()
	
	self.frames["DrawingBoardBoard".. self.active.."Button"]:LockHighlight()
    self.frames["DrawingBoardMainFrame"].title:SetText(self.boards[self.active].title)
end




function DrawingBoard:RenameBoard(boardnumber, newname)
    self.boards[boardnumber].title = newname
    
    local b = self.frames["DrawingBoardBoard"..boardnumber.."Button"]
    b:SetText(newname)
    b:SetWidth(b:GetFontString():GetStringWidth()+16)
	
    if self.active == boardnumber then
	self.frames["DrawingBoardMainFrame"].title:SetText(newname)
    end	
end



function DrawingBoard:SortBoardButtons()
    local y = 0
    local lastnumber = -1
    for number, _ in pairs(self.boards) do
        self.frames["DrawingBoardBoard"..number.."Button"]:ClearAllPoints()
        self.frames["DrawingBoardBoard"..number.."Button"]:SetPoint("TOPRIGHT", self.frames["DrawingBoardMainFrame"], "TOPLEFT", -2, -y)
        y= y + self.frames["DrawingBoardBoard"..number.."Button"]:GetHeight() + 2
        lastnumber = number
    end
    self.frames["DrawingBoardNewBoardButton"]:ClearAllPoints()
    if lastnumber ~= -1 then
        self.frames["DrawingBoardNewBoardButton"]:SetPoint("TOPRIGHT", self.frames["DrawingBoardBoard"..lastnumber.."Button"], "BOTTOMRIGHT", 0, -2)
    else
		self.frames["DrawingBoardNewBoardButton"]:SetPoint("TOPRIGHT", self.frames["DrawingBoardMainFrame"], "TOPLEFT", -2, 0)
    end
end


function DrawingBoard:Broadcast(boardID)
    local board = self.boards[boardID]
    local message
    local first
    local assignments = {}
  -- ToDo : Ausgabe sortieren.
    self:SendMessage(board.title, board.channel)
    for name,tag in pairs(board.tags) do
	if not tag.attached_to then
	    message = board.prefix ..tag.text ..": "
	    first = true
	    for _, tag2 in pairs(board.tags) do
			if tag2.attached_to == tag.id then
				if not first then message = message .. ", " end
				first = false
				message = message .. tag2.text
			end
	    end
	    self:SendMessage(message, board.channel)
	end
    end
end


-- ********** Create Frames

function DrawingBoard:CreateMainFrames()
    local f = CreateFrame("Frame", "DrawingBoardMainFrame", UIParent)
    f:SetFrameStrata("FULLSCREEN")
    f:SetWidth(700)
    f:SetHeight(500)
    f:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 32, edgeSize = 8, 
                                            insets = { left = 0, right = 0, top = 0, bottom = 0 }})
    f:SetBackdropColor(0, 0, 0, 0.8)
    f:SetClampedToScreen(true)
    f:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 150, -120)
    
    f.title = f:CreateFontString(nil, "ARTWORK")
    f.title:SetFontObject(GameFontNormalLarge)
    f.title:SetJustifyH("LEFT")
    f.title:SetPoint("TOPLEFT",6,-6)
    
    self.frames["DrawingBoardMainFrame"] = f
    
    local b = CreateFrame("Button", "DrawingBoardCustomButton", f, "OptionsButtonTemplate")
    b:SetFrameStrata("FULLSCREEN")
    b:SetPoint("TOPRIGHT", f, "BOTTOMRIGHT", 0, -2)
    b:SetText(L["Custom Assignment"])
    b:SetWidth(b:GetFontString():GetStringWidth()+16)
    b:SetScript("OnClick", function () DrawingBoard:ShowCustomDialog() end)
    self.frames["DrawingBoardCustomButton"] = b
    
    local b = CreateFrame("Button", "DrawingBoardCloseButton", f, "OptionsButtonTemplate")
    b:SetFrameStrata("FULLSCREEN")
    b:SetPoint("BOTTOMRIGHT", f, "TOPRIGHT", 0, 2)
    b:SetText("X")
    b:SetWidth(b:GetHeight()+10)
    b:SetScript("OnClick", function () DrawingBoard:Close() end)
    self.frames["DrawingBoardCloseButton"] = b
    
    local b = CreateFrame("Button", "DrawingBoardDeleteBoardButton", f, "OptionsButtonTemplate")
    b:SetFrameStrata("FULLSCREEN")
    b:SetPoint("RIGHT", self.frames["DrawingBoardCloseButton"], "LEFT", -2, 0)
    b:SetText(L["Delete Board"])
    b:SetWidth(b:GetFontString():GetStringWidth()+16)
    b:SetScript("OnClick", function () DrawingBoard:DeleteBoard() end)
    self.frames["DrawingBoardDeleteBoardButton"] = b
    
    
    local b = CreateFrame("Button", "DrawingBoardBoardOptionsButton", f, "OptionsButtonTemplate")
    b:SetFrameStrata("FULLSCREEN")
    b:SetPoint("BOTTOMLEFT", self.frames["DrawingBoardMainFrame"], "TOPLEFT", 0, 2)
    b:SetText(L["Board Options"])
    b:SetWidth(b:GetFontString():GetStringWidth()+16)
    b:SetScript("OnClick", function () DrawingBoard:ShowBoardOptionsDialog() end)
    self.frames["DrawingBoardBoardOptionsButton"] = b
    
    local b = CreateFrame("Button", "DrawingBoardNewBoardButton", f, "OptionsButtonTemplate")
    b:SetFrameStrata("FULLSCREEN")
    b:SetText("+")
    b:SetWidth(b:GetHeight()+10)
    b:SetScript("OnClick", function () self:ShowBoard(DrawingBoard:NewBoard()) end)
    self.frames["DrawingBoardNewBoardButton"] = b
    
    local b = CreateFrame("Button", "DrawingBoardBroadcastButton", f, "OptionsButtonTemplate")
    b:SetFrameStrata("FULLSCREEN")
    b:SetPoint("LEFT", self.frames["DrawingBoardBoardOptionsButton"], "RIGHT", 2, 0)
    b:SetText(L["Broadcast Board"])
    b:SetWidth(b:GetFontString():GetStringWidth()+16)
    b:SetScript("OnClick", function () DrawingBoard:Broadcast(DrawingBoard.active) end)
    self.frames["DrawingBoardBroadcastButton"] = b
	
	local b = CreateFrame("Button", "DrawingBoardSendButton", f, "OptionsButtonTemplate")
    b:SetFrameStrata("FULLSCREEN")
    b:SetPoint("LEFT", self.frames["DrawingBoardBroadcastButton"], "RIGHT", 2, 0)
    b:SetText(L["Send Board"])
    b:SetWidth(b:GetFontString():GetStringWidth()+16)
    b:SetScript("OnClick", function () DrawingBoard:ShowSendDialogue() end)
    self.frames["DrawingBoardSendButton"] = b
end


function DrawingBoard:CreateBoardButtons()
	local b
	for number, board in pairs(self.boards) do
		b = CreateFrame("Button", "DrawingBoardBoard"..number.."Button", f, "OptionsButtonTemplate")
		b:SetFrameStrata("FULLSCREEN")
		b:SetScript("OnClick", function () DrawingBoard:ShowBoard(number) end)
		b:Show()
		self.frames["DrawingBoardBoard"..number.."Button"] = b
		self:RenameBoard(number, board.title)
	end
	self:SortBoardButtons()
end

function DrawingBoard:ShowBoardOptionsDialog()
    local d = LibStub("AceGUI-3.0"):Create("Frame")
    local title = LibStub("AceGUI-3.0"):Create("EditBox")
    local prefix = LibStub("AceGUI-3.0"):Create("EditBox")
    local channel = LibStub("AceGUI-3.0"):Create("EditBox")
    local default = LibStub("AceGUI-3.0"):Create("Button")

    
    d:SetTitle(L["Board Options"])
    d:SetWidth(400)
    d:SetHeight(225)
    
    title:SetText(DrawingBoard.db.profile.boards[DrawingBoard.active].title)
    title:SetLabel(L["Title"])
    title:SetCallback("OnEnterPressed", function(self, event, text)
					DrawingBoard.db.profile.boards[DrawingBoard.active].title = text
					DrawingBoard.frames["DrawingBoardBoard"..DrawingBoard.boards[DrawingBoard.active].number.."Button"]:SetText(text)
					DrawingBoard.frames["DrawingBoardBoard"..DrawingBoard.boards[DrawingBoard.active].number.."Button"]:SetWidth(DrawingBoard.frames["DrawingBoardBoard"..DrawingBoard.db.profile.boards[DrawingBoard.active].number.."Button"]:GetFontString():GetWidth()+16)
					DrawingBoard.frames["DrawingBoardMainFrame"].title:SetText(text)
				    end)
				    
    prefix:SetText(DrawingBoard.db.profile.boards[DrawingBoard.active].prefix)
    prefix:SetLabel(L["Prefix"])
    prefix:SetCallback("OnEnterPressed", function(self, event, text)
					DrawingBoard.db.profile.boards[DrawingBoard.active].prefix = text
				    end)
    
    channel:SetText(DrawingBoard.db.profile.boards[DrawingBoard.active].channel)
    channel:SetLabel(L["Channel"])
    channel:SetCallback("OnEnterPressed", function(self, event, text)
					DrawingBoard.db.profile.boards[DrawingBoard.active].channel = text
				    end)
				    
    default:SetText(L["Set to default"])
    default:SetCallback("OnClick", function (self, event)
					DrawingBoard.db.profile.bc_title = DrawingBoard.boards[DrawingBoard.active].title
					DrawingBoard.db.profile.bc_prefix = DrawingBoard.boards[DrawingBoard.active].prefix
					DrawingBoard.db.profile.bc_channel = DrawingBoard.boards[DrawingBoard.active].channel
				    end)
        	    
    d:AddChild(title)
    d:AddChild(prefix)
    d:AddChild(channel)
    d:AddChild(default)
    d:Show()
end


function DrawingBoard:ShowCustomDialog()
    local d = LibStub("AceGUI-3.0"):Create("Frame")
    local e = LibStub("AceGUI-3.0"):Create("EditBox")
    
    d:SetWidth(400)
    d:SetHeight(200)
    
    d:SetTitle(L["Custom Assignment"])
    e:SetCallback("OnEnterPressed", function(self, event, text)
					DrawingBoard:CreateButton(text)
					d:SetStatusText("Custom Button "..text.." created")
					e:SetText("")
				    end)

    d:AddChild(e)
    d:Show()
    e.editbox:SetFocus()
end

function DrawingBoard:CreateButton(text, id)
	if not text then return end
    if not id then
        id = 0
        while self.tagframes[id] do
            id = math.random(100000)
        end
    end
    local type = "custom" 
	if string.find(text,"{rt") then type = "rt" end
	local i, class = DrawingBoard:RaidSortIndex(text)
	if i then type = "player" end
    if type == "rt" then
		i =  string.sub(text, string.find(text, '%d'))
	end

    local f = CreateFrame("Frame", nil, UIParent)
    f:SetFrameStrata("FULLSCREEN")
    f:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 8, 
                                            insets = { left = 0, right = 0, top = 0, bottom = 0 }})
    f:SetBackdropColor(0, 0, 0, 1)
    f:SetClampedToScreen(true)
    
    f.text = f:CreateFontString(nil, "ARTWORK")
    f.text:SetFontObject(GameFontNormal)
    f.text:SetJustifyH("LEFT")
    f.text:SetPoint("LEFT",6,0)
    if type ~= "rt" then f.text:SetText(text) else f.text:SetText("") end
    
    f:SetWidth(f.text:GetStringWidth()+12)
    f:SetHeight(f.text:GetStringHeight()+7)
    
    if type == "custom" then f.text:SetTextColor(0.6, 0.6, 0.6, 1) end
    if type == "player" then
        f.text:SetTextColor(RAID_CLASS_COLORS[class].r,RAID_CLASS_COLORS[class].g,RAID_CLASS_COLORS[class].b,1)
    end
	

    
    f:SetMovable(true)
    f:RegisterForDrag("LeftButton")
    f:EnableMouse(true)
    f:SetScript("OnDragStart", f.StartMoving)
    f:SetScript("OnDragStop", function () f:StopMovingOrSizing(); self:DropButton(id) end )
	
    if type == "rt" then
		f.texture = f:CreateTexture(nil, "OVERLAY")
		f.texture:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
		f.texture:SetPoint("TOPLEFT", f, "TOPLEFT", 2, -2)
		f.texture:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -2, 2)
		SetRaidTargetIconTexture(f.texture,i)
		f:SetWidth(25)
		f:SetHeight(25)
		f.ToDefaultPosition = function (self) 
			self:ClearAllPoints()
			f:SetPoint("TOPLEFT", DrawingBoard.frames["DrawingBoardMainFrame"], "BOTTOMLEFT", (i-1)*(f:GetWidth()+2), -2)
		end
	end
    if type == "custom" then
        f.ToDefaultPosition = function (self) 
			self:ClearAllPoints()
			self:SetPoint("TOPLEFT", DrawingBoard.frames["DrawingBoardMainFrame"], "TOPLEFT", 350, -505) 
		end
    end
	if type == "player" then
		f.ToDefaultPosition = function (self)
			self:ClearAllPoints()
			self:SetPoint("TOPLEFT", DrawingBoard.frames["DrawingBoardMainFrame"], "TOPRIGHT", 2, 20-20*i) 
		end
	end
	
	f.GetRelativePoint = function (self)
		local scale1 = self:GetScale()
		local scale2 = DrawingBoard.frames["DrawingBoardMainFrame"]:GetScale()
		local left1, bottom1, width1, height1 = self:GetRect()
		left1, bottom1, width1, height1 = left1*scale1, bottom1*scale1, width1*scale1, height1*scale1
		local left2, bottom2, width2, height2 = DrawingBoard.frames["DrawingBoardMainFrame"]:GetRect()
		left2, bottom2, width2, height2 = left2*scale2, bottom2*scale2, width2*scale2, height2*scale2
		return left1+width1/2.0-left2-width2/2.0, bottom1+height1/2.0-bottom2-height2/2.0
	end
	f.SetRelativePoint = function (self, x, y)
		local scale1 = self:GetScale()
		local scale2 = DrawingBoard.frames["DrawingBoardMainFrame"]:GetScale()
		self:ClearAllPoints()
		self:SetPoint("CENTER", DrawingBoard.frames["DrawingBoardMainFrame"], "CENTER", x*scale2/scale1, y*scale2/scale1)
	end
    f.GetText = function () return text end
    f.GetID = function () return id end
    f.attached = {}
    f:ToDefaultPosition()  
	f.GetType = function (self) return type end 
    self.tagframes[id] = f
    
	if self.frames["DrawingBoardMainFrame"]:IsShown() then
		f:Show()
	else
		f:Hide()
	end
	
    return f
end


function DrawingBoard:RaidSortIndex(text)
	if not (GetNumRaidMembers() > 0) then return end
	local classlist = {}
    local class_names = {}
    local class
    local name
	
    for i=1,GetNumRaidMembers() do
		_, class = UnitClass("raid"..i)
		name = UnitName("raid"..i)
		if not class_names[class] then
			class_names[class] = {}
			table.insert(classlist, class)
		end
		table.insert(class_names[class], name)
    end
    
    table.sort(classlist)
    for _, names in pairs(class_names) do
		table.sort(names)
    end
    
	local i = 1
	for _, class in pairs(classlist) do
		for _, name in pairs(class_names[class]) do
			if name == text then return i, class end
			i = i + 1
		end
	end
end




-- ******* Button functions


function DrawingBoard:GetChildren(buttonID)
    if self.boards[self.active].tags[buttonID].attached_to then
        return nil
    end
    local result = {}
    for id, tag in pairs(self.boards[self.active].tags) do
        if tag.attached_to == buttonID then
            table.insert(result, id)
        end
    end
    return result
end


function DrawingBoard:ResetButton(buttonID)
    local self = DrawingBoard
    local button = self.tagframes[buttonID]
    local data =  self.boards[self.active].tags[buttonID] or {}
    local attached_to = data.attached_to
    
	if button:GetType() == "custom" then
		self:DeleteButton(buttonID)
	else
		local count = 0
		for _, button2 in pairs(self.tagframes) do
			if button2:GetText() == button:GetText() then
				count = count +1
			end
		end
		if count > 1 then
			self:DeleteButton(buttonID)
		else
			button:SetScale(1)
			button:ToDefaultPosition()
		end
	end
	
    for id, tag in pairs(self.boards[self.active].tags) do
        if tag.attached_to == buttonID then
            self:ResetButton(id)
        end
    end
	
	self.boards[self.active].tags[buttonID] = nil
    if attached_to then
		self:ArrangeChildren(attached_to)
    end
end


function DrawingBoard:UpdateButton(buttonID)
	local button = self.tagframes[buttonID]
	if not button then self:Debug("No button with id " ,buttonID, " to update")  return end
	local data = self.boards[self.active].tags[buttonID]
	if button:GetType() == "rt" then return end
	local oldtype = button:GetType()
	local type = "custom"
	local i, class = self:RaidSortIndex(button:GetText())
	if i then type = "player" end
	
	if type == "custom" then 
		if oldtype == "player" and not data then 
			DrawingBoard:DeleteButton(buttonID)
		end
		button.text:SetTextColor(0.6, 0.6, 0.6, 1) 
		button.ToDefaultPosition = function (self) 
			self:ClearAllPoints()
			self:SetPoint("TOPLEFT", DrawingBoard.frames["DrawingBoardMainFrame"], "TOPLEFT", 350, -505) 
		end
	end
    if type == "player" then
        button.text:SetTextColor(RAID_CLASS_COLORS[class].r,RAID_CLASS_COLORS[class].g,RAID_CLASS_COLORS[class].b,1)
		button.ToDefaultPosition = function (self)
			self:ClearAllPoints()
			self:SetPoint("TOPLEFT", DrawingBoard.frames["DrawingBoardMainFrame"], "TOPRIGHT", 2, 20-20*i) 
		end
		if not data then button:ToDefaultPosition() end
    end
	button.GetType = function () return type end
end



function DrawingBoard:DeleteButton(buttonID)
    self.tagframes[buttonID]:Hide()
	self.tagframes[buttonID] = nil
end


function DrawingBoard:DropButton(buttonID)
    local self = DrawingBoard
    local button = self.tagframes[buttonID]
    local data = self.boards[self.active].tags
    
    if self:CheckCollision(self.frames["DrawingBoardMainFrame"], button) then
        if not data[buttonID] then
            data[buttonID] = {}
            data[buttonID].id = buttonID
			data[buttonID].text = button:GetText()
        end
		
		if data[buttonID].attached_to then
			local temp = data[buttonID].attached_to
			data[buttonID].attached_to = nil
			self:ArrangeChildren(temp)
		end
    
		local assignment = nil
		for id, tag in pairs(data) do
			if not tag.attached_to and id ~= buttonID then
				if self:CheckCollision(self.tagframes[id], button) then
					assignment = id
				end
			end
		end
		if assignment then
			self:AttachButton(buttonID, assignment)
		else
			local x, y = button:GetRelativePoint()
			data[buttonID].x = x
			data[buttonID].y = y
			
			button:SetScale(1.5)
			button:SetRelativePoint(x, y) 
		end
    else
		self:ResetButton(buttonID)
    end
end


function DrawingBoard:AttachButton(buttonID, assignmentID)
    self.tagframes[buttonID]:SetScale(1)
    self.boards[self.active].tags[buttonID].attached_to = assignmentID
	for id,  tag in pairs(self.boards[self.active].tags) do
		if tag.attached_to == buttonID then
			self:ResetButton(id)
		end
	end
    self:ArrangeChildren(assignmentID)
end


function DrawingBoard:ArrangeChildren(buttonID)
    local button = self.tagframes[buttonID]
    local data = self.boards[self.active].tags
    if not button then
		return
	end
    -- Calculate new positions
    local n=0
    local attached = {}
    for _, tag in pairs(data) do
	if tag.attached_to == buttonID then
            n = n + 1
            table.insert(attached, tag.id) 
        end
    end
    
    local r = 20
    repeat
		local i = 0
		for _, id in pairs(attached) do
			self.tagframes[id]:ClearAllPoints()
			self.tagframes[id]:SetPoint("CENTER", button, "CENTER", math.sin(2*math.pi*i/n)*r, math.cos(2*math.pi*i/n)*r)
			i = i + 1
		end
		r = r + 5
    until self:ButtonsPretty(buttonID) or r == 1000
end


function DrawingBoard:ButtonsPretty(buttonID)
    local data = self.boards[self.active].tags
    for _, tag in pairs(data) do
        if tag.attached_to == buttonID then
            if self:CheckCollision(self.tagframes[buttonID], self.tagframes[tag.id]) then
                return false
            end
        end
    end
    
    for id1, tag1 in pairs(data) do
		for id2,tag2 in pairs(data) do
			if id1 ~= id2 and tag1.attached_to == buttonID and tag2.attached_to == buttonID  and  self:CheckCollision(self.tagframes[id1], self.tagframes[id2]) then
			   return false
			end
		end	
    end
 
    return true
end


function DrawingBoard:HasButton(text)
	for _, button in pairs(self.tagframes) do
		if button:GetText() == text then return true end
	end
	return false
end

-- ********* Helpers

function DrawingBoard:Debug(...)
    if self.debug then self:Print(...) end
end


function DrawingBoard:CheckCollision(button1, button2)
	if not button1 or not button2 then
		return false
	end
    local scale1 = button1:GetScale()
    local scale2 = button2:GetScale()
    
    local left1, bottom1, width1, height1 = button1:GetRect()
    left1, bottom1, width1, height1 = left1*scale1, bottom1*scale1, width1*scale1, height1*scale1
    local left2, bottom2, width2, height2 = button2:GetRect()
    left2, bottom2, width2, height2 = left2*scale2, bottom2*scale2, width2*scale2, height2*scale2
    
    if left1 < left2+width2 and bottom1+height1 > bottom2 and left1+width1 > left2 and bottom1 < bottom2+height2 then
		return true
    else
		return false
    end
end

function DrawingBoard:SendMessage(text, channel, target)
    if channel == "" or channel == nil then
	self:Print(text);
    elseif tonumber(channel) then
	ChatThrottleLib:SendChatMessage("NORMAL", "DrawingBoard", text, "CHANNEL", nil, tonumber(channel))
    elseif channel == "WHISPER" then
	ChatThrottleLib:SendChatMessage("NORMAL", "DrawingBoard", text, channel,nil, target)
    else
	ChatThrottleLib:SendChatMessage("NORMAL", "DrawingBoard", text, channel)
    end
end
